/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */

import { DebugUtil } from '../../apps/tm-viewer';
import { CustomElementProperty } from '../../mol-model-props/common/custom-element-property';
import { Model, ElementIndex, StructureElement } from '../../mol-model/structure';
import { Loci } from '../../mol-model/loci';
import { LociLabel, LociLabelProvider } from '../../mol-plugin-state/manager/loci-label';
import { Representation } from '../../mol-repr/representation';
import { TmDetDescriptorCache } from './prop';
import { createResidueListsPerChain } from './tmdet-color-theme';
import { ChainList } from './types';
import { OrderedSet } from '../../mol-data/int';

const siteLabels = [
    "Side1",
    "Side2",
    "TM alpha",
    "TM beta",
    "TM re-entrant loop",
    "Interfacial Helix",
    "Unknown localization",
    "Membrane Inside"
];
const DefaultResidueLabel = 6; // Unknown localization


export const LabeledResidues = CustomElementProperty.create<number>({
    label: 'TMDet Topology Site Labels',
    name: 'tmdet-topology-site-labels',
    getData(model: Model) {
        DebugUtil.log('start getData', model.label, model.entryId);

        const map = new Map<ElementIndex, number>();
        const descriptor = TmDetDescriptorCache.get(model.entryId);
        if (!descriptor) {
            return { value: map };
        }

        const getSiteId = function(chains: ChainList, residueId: string): number {
            // TODO: integrate with the coloring version
            let label = DefaultResidueLabel;

            for (let chain of chains) {
                const residue = chain.residues.filter((res) => res.authId === residueId)[0];
                if (residue) {
                    label = residue.siteId;
                    break;
                }
            }
            //const chain = chains.filter((chain) => chain.chainId === chainId)[0];
            // if (chain) {
            //     const residue = chain.residues.filter((res) => res.authId === residueId)[0];
            //     if (residue) {
            //         label = siteLabels[residue.siteId];
            //     }
            // }

            return label;
        };

        const chainList = createResidueListsPerChain(descriptor.chains);
        DebugUtil.log('getData', model.atomicHierarchy.chainAtomSegments);
        // DebugUtil.log('getData: auth_comp_id', model.atomicHierarchy.atoms.auth_comp_id.toArray());
        const residueIndex = model.atomicHierarchy.residueAtomSegments.index;
        for (let i = 0, _i = model.atomicHierarchy.atoms._rowCount; i < _i; i++) {
            const residueIdx = residueIndex[i];
            const authId = model.atomicHierarchy.residues.auth_seq_id.value(residueIdx).toString();
    
            map.set(i as ElementIndex, getSiteId(chainList, authId));
        }
        DebugUtil.log('end of getData');
        return { value: map };
    },
    getLabel(e) {
        return siteLabels[e];
    }
});


export const TmLabelProvider: LociLabelProvider = {
    label: (loci: Loci): LociLabel => {
        if (loci.kind == 'element-loci') {
            DebugUtil.log('LabelProvider:', loci.elements[0].unit.model.atomicHierarchy.chains.auth_asym_id.toArray());
            DebugUtil.log('LabelProvider:', loci.elements[0].unit.model.atomicHierarchy.chains.label_asym_id.toArray());
            DebugUtil.log('LabelProvider:', loci.elements[0].unit.model.atomicHierarchy.chains.label_entity_id.toArray());
            DebugUtil.log('IsLocation:', loci.structure.elementLocations());
            DebugUtil.log('start:', OrderedSet.start(loci.elements[0].indices));
        }
        return "hello";
    }
}
